#!/usr/bin/env python
# encoding: utf-8
import gettext
from pathlib import Path

from .battleship import Game, Board, Ship  # noqa
from .version import __version__  # noqa


BASE_DIR = Path(__file__).absolute().parents[1]
LOCALE_DIR = BASE_DIR / "locale"

language = gettext.translation('battleship', localedir=str(LOCALE_DIR), languages=['pt_BR', 'en'])
language.install()


__all__ = ['Game', 'Board', 'Ship', '__version__', 'language', 'BASE_DIR']



#!/usr/bin/env python
# encoding: utf-8
import os
import sys
from string import ascii_lowercase

from .battleship import Game
from .exceptions import InvalidPosition
from . import language

_ = language.gettext

LETTERS = {letter: str(index) for index, letter in enumerate(ascii_lowercase, start=0)}
NUMBERS = {index: letter for index, letter in enumerate(ascii_lowercase, start=0)}


def print_board(game):
    os.system('clear')
    print(_('\nPoints: '), game.points, end="     ")
    print(_('Shots Available: '), game.shots, end="     ")
    print(_('Time elapsed: '), '{} sec'.format(int(game.time_elapsed)), end="\n")
    print('-' * 90)
    # Table Header
    print('| ', end='')
    print('{:<3}'.format('-'), end='')
    for i in range(game.COLS):
        print('| ', end='')
        print('{:<3}'.format(i), end='')
    print('| ', end='\n')
    print('-' * 90)
    # Table Body
    for i, row in enumerate(game.matrix):
        print('| ', end='')
        print('{:<2}'.format(NUMBERS[i]), end=" | ")
        for col in row:
            if col['shooted']:
                if not col['ship']:
                    print('{:<2}'.format('O'), end=" | ")
                else:
                    print('{:<2}'.format(col['ship'].initials), end=" | ")
            else:
                print('{:<2}'.format('.'), end=" | ")

        print()

    print('-' * 90)
    print(_('Labels'))
    print('-' * 7)
    for ship in game.ships:
        text = _('<{}> \t Initials: {} \t| Size: {} \t| Hits: {} | sunk: {}')
        print(text.format(
            ship.name.title(),
            ship.initials,
            ship.length,
            ship.hits,
            'yes' if ship.sink else 'no'
        ))

    print()
    print('-' * 90)


def print_ship_hit(ship_hit, points, shots, time_elapsed):
    os.system('clear')
    print(_('\nPoints: '), points, end="     ")
    print(_('Shots Available: '), shots, end="     ")
    print(_('Time elapsed: '), '{} sec'.format(int(time_elapsed)), end="\n")
    print('-' * 90)
    print('\n\n\n\n')
    if ship_hit.sink:
        print(_('\t\t\t You Destroy a {}').format(ship_hit.name.title()))
    else:
        print(_('\t\t\t You Hit a {}').format(ship_hit.name.title()))

    print('\n\n\n\n')
    input(_("Press Enter to continue..."))


def print_status(game):
    os.system('clear')
    print('\nPoints: ', game.points, end="     ")
    print('Lost Shots: ', game.lost_shot, end="     ")
    print('Right Shots: ', game.lost_shot, end="     ")
    print('Shots Missing: ', game.shots, end="\n")
    print('-' * 90)
    print('Sunken Ships: ', game.sunken_ships, end="     ")
    print('Missing Ships: ', game.total_ships - game.sunken_ships, end="   ")
    print('Time elapsed: {} sec'.format(int(game.time_elapsed)), end="\n")
    print('-' * 90)
    for s in game.ships:
        print('<{}> \t Sunk: {} \t| Hits: {}'.format(
            s.name.title(),
            'yes' if s.sink else 'no',
            s.hits
        ))

    print('-' * 90)


def main():
    game = Game()
    print_board(game)

    while True:
        while True:
            try:
                print(_('Press CTRL+C to exit ...'))
                result = input(_('Choose your coordinates. Ex: a1, b15: c10 \n\nCoordinates: ')).strip()

                x = int(LETTERS[result[0].lower()])
                y = int(result[1:].strip())

                if (x > 16 and x < 0) or (y > 16 and y < 0):
                    raise InvalidPosition()

                is_valid, ship = game.play(x, y)
                if is_valid:
                    print_ship_hit(ship, game.points, game.shots, game.time_elapsed)
                    break

                print_board(game)

                if game.end_game():
                    sys.exit(print_status(game))

            except Exception as exc:
                os.system('clear')
                print(str(exc))
                input(_('\n\n\n\t\t\tYou can only use letters with numbers. Ex: a1, c10, etc...'))
                print_board(game)
            except KeyboardInterrupt:
                sys.exit(print_status(game))

        print_board(game)

main()



#!/usr/bin/env python
# encoding: utf-8
from datetime import datetime
import random
from copy import deepcopy

from .exceptions import InvalidPosition


class Ship:
    def __init__(self, name, length, points, initials):
        self.direction = ''
        self.hit_positions = []
        self.initials = initials
        self.length = length
        self.name = name
        self.sink = False
        self.points = points

    @property
    def hits(self):
        return len([x['hit'] for x in self.hit_positions if x['hit']])

    @property
    def is_destroyed(self):
        results = []
        for position in self.hit_positions:
            results.append(
                position['hit']
            )
        return all(results)

    @property
    def positions(self):
        return len(self.hit_positions)

    def add_location(self, x, y, hit):
        if len(self.hit_positions) + 1 > self.length:
            text = 'Ship has more hit positions {} than Lenght: {}'.format(
                len(self.hit_positions) + 1,
                self.length
            )
            raise InvalidPosition(text)

        self.hit_positions.append({
            'x': x,
            'y': y,
            'hit': hit
        })

    def hit(self, x, y):
        for pos in self.hit_positions:
            if (x, y) == (pos['x'], pos['y']):
                pos['hit'] = True

    def __str__(self):
        return self.name.title()

    def __repr__(self):
        return "<Ship: {}>".format(self.__str__())


class Board:
    COLS = 17
    ROWS = 17
    DEFAULT_VALUE = {
        'ship': None,
        'shooted': False,
    }
    SHIP_TYPES = [
        {'name': 'carrier', 'length': 5, 'points': 250, 'initials': 'CA'},
        {'name': 'battleship', 'length': 4, 'points': 200, 'initials': 'BT'},
        {'name': 'cruiser', 'length': 3, 'points': 150, 'initials': 'CR'},
        {'name': 'destroyer', 'length': 3, 'points': 150, 'initials': 'DT'},
        {'name': 'submarine', 'length': 2, 'points': 100, 'initials': 'SB'},
        {'name': 'frigate', 'length': 2, 'points': 100, 'initials': 'FR'},
    ]

    def __init__(self):
        self.matrix = self.clear_board()
        self.sunken_ships = 0
        self.ships = []
        self.total_hits = 0
        self._init_ships()

    @property
    def total_available_ships(self):
        return len(self.ships)

    @property
    def total_ships(self):
        return len([ship for ship in self.ships if ship.positions])

    def _init_ships(self):
        for _, ship_type in enumerate(self.SHIP_TYPES):
            self.ships.append(Ship(**ship_type))

    def _add_ship(self, ship):
        direction = random.choice(['v', 'h'])
        ship.direction = direction
        length = ship.length

        if direction == 'h':
            x = random.randint(0, self.COLS - length)
            end_position = (x + length)

            for col in range(x, end_position):
                if not self.is_available(x, col):
                    return False

                ship.add_location(x, col, False)

                value = deepcopy(self.DEFAULT_VALUE)
                value['ship'] = ship
                self.matrix[x][col] = value
        else:
            y = random.randint(0, self.ROWS - length)
            end_position = (y + length)

            for row in range(y, end_position):
                if not self.is_available(row, y):
                    return False

                ship.add_location(row, y, False)
                value = deepcopy(self.DEFAULT_VALUE)
                value['ship'] = ship
                self.matrix[row][y] = value

        return True

    def is_finished(self):
        results = []
        for ship in self.ships:
            results.append(
                ship.is_destroyed
            )
        return all(results)

    def _remove_ship(self, ship):
        for position in ship.hit_positions:
            self.matrix[position['x']][position['y']] = deepcopy(self.DEFAULT_VALUE)

    def add_ship(self, ship):
        added = self._add_ship(ship)
        if not added:
            self._remove_ship(ship)
            return self.add_ship(ship)
        return added

    def build_fleet(self):
        for ship in self.ships:
            self.add_ship(ship)

    def clear_board(self):
        default_value = deepcopy(self.DEFAULT_VALUE)
        return [[default_value for _ in range(self.COLS)] for _ in range(self.ROWS)]

    def is_available(self, x, y):
        return self.matrix[x][y] == deepcopy(self.DEFAULT_VALUE)

    def shot(self, x, y):
        position = self.matrix[x][y].copy()

        if position['shooted']:
            return False, None

        position.update(shooted=True)
        if not position['ship']:
            self.matrix[x][y] = position
            return False, None

        position['ship'].hit(x, y)

        self.total_hits += 1
        if position['ship'].is_destroyed:
            self.sunken_ships += 1
            position['ship'].sink = True

        self.matrix[x][y] = position
        return True, position['ship']


class Game(Board):
    def __init__(self):
        super().__init__()
        self.start_time = datetime.now()
        self.shots = 50
        self.points = 0
        self.lost_shot = 0
        self.right_shot = 0

        # Building Fleet
        self.build_fleet()

    @property
    def time_elapsed(self):
        end_time = datetime.now()
        return (end_time - self.start_time).total_seconds()

    def play(self, x, y):

        if self.matrix[x][y]['shooted']:
            return False, None
        else:
            self.shots -= 1

            hit, ship = self.shot(x, y)

            if not hit:
                self.lost_shot += 1
                return False, None

            self.right_shot += 1
            self.points += ship.points

            if self.is_finished() or self.shots == 0:
                raise Exception('Finish')

            return True, ship

    def end_game(self):
        if self.is_finished() or self.shots == 0:
            return True
        return False
